# -*- coding: utf-8 -*-

# core.py
# This file is part of qarte-4
#    
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Copyright: 2016-2020 Vincent Vande Vyvre
# Licence: GPL3
# Home page: https://launchpad.net/qarte

import os
import sys
import json
import pickle
import time
import locale

import logging
lgg = logging.getLogger(__name__)

import gettext
LOC_PATH = '/usr/share/locale'
gettext.install('qarte', LOC_PATH, True)
# For testing a specific translation, comment the previous line,
# uncomment the two next lines and set the language code.
#tr = gettext.translation('qarte', LOC_PATH, languages=['de'])
#tr.install()

from data import VERSION, VERSION_STR
from artetv import ArteTV
from arteconcert import ArteConcert
from gui.mainui import MainUi
from settings import Settings
from gui.warning import WarningBox
from loadingscheduler import LoadingScheduler
from about import About

try:
    from shutil import disk_usage
except ImportError:
    from diskusage import disk_usage

from PyQt5.QtCore import QCoreApplication, QLocale
from PyQt5.QtWidgets import QApplication

LOC = QLocale.system().name()
try:
    LANG, c = LOC.split('_')
except:
    LANG = 'fr'

if LANG not in ['fr', 'de']:
    LANG = 'fr'


class Core:
    """Define the main class of Qarte.

    """
    def __init__(self, args):
        self.set_environment(args)
        self.is_interact_with_dialog = False
        self.build_main_window()
        self.artetv = ArteTV(self)
        self.ui.tv_basket.driver = self.artetv
        live = self.ui.live_pg
        live.viewer.cfg = self.cfg
        self.artetv.parsingFinished.connect(self.set_videos_list)
        self.artetv.config_parser()
        self.artelive = ArteConcert(self)
        live.live_basket.driver = self.artelive
        self.artelive.parsingFinished.connect(self.set_concerts_list)
        # The viewer need a ref to artelive
        #live.viewer.al = self.artelive
        self.ui.configure_signals()

    def build_main_window(self):
        lgg.info('Build main window')
        self.ui = MainUi(self, self.cfg["size"])
        self.ui.show()
        QCoreApplication.processEvents()
        self.ui.resize_splitters()

    def set_environment(self, args):
        """Build the working environment.

        """
        self.workspace = self.define_workspace()
        self.cfg = self.get_config(args)
        lgg.info("with video player: %s" % self.cfg['video_player'])

    def get_config(self, args):
        """Load the user's preferences.

        a default config is used.
        """
        if os.path.isfile(self.workspace['config']):
            cfg = self.load_config()

        else:
            cfg = self.create_config()

        cfg['version'] = VERSION
        cfg['live_quality'] = 'HTTP_EQ_1'
        if not 'add_date' in cfg:
            cfg['add_date'] = False

        if not 'video_player' in cfg:
            cfg['video_player'] = True

        if "--no-player" in args:
            cfg['video_player'] = False

        elif "--with-player" in args:
            cfg['video_player'] = True

        return cfg

    def create_config(self):
        """Create the config with default values.

        """
        lgg.info('Config not found, use default')
        from data import DEFAULT_PREFS
        idx = [None, 'fr', 'de'].index(LANG)
        DEFAULT_PREFS['tv_quality'] = 'HTTPS_MP4_EQ_%s' % idx
        return DEFAULT_PREFS

    def read_old_config(self, file_):
        """Load an old user's preferences file.

        Args:
        file_ -- path of the file
        """
        with open(file_, 'rb') as inf:
            try:
                cfg = pickle.load(inf)
            except:
                # Use default
                from data import DEFAULT_PREFS as cfg

        return cfg

    def load_config(self):
        """Read the user config.

        """
        path = self.workspace['config']
        lgg.info('Load config from: %s' % path)
        try:
            with open(path, 'r') as objfile:
                cnt = objfile.read()
                return json.loads(cnt)
        except Exception as why:
            lgg.warning("Read error from: {0},".format(path))
            lgg.warning("reason: {0}.".format(why))
            return self.create_config()

    def save_user_config(self):
        """Save the user's preferences.

        """
        lgg.info('Save user config')
        self.cfg['size'] = self.ui.width(), self.ui.height()
        jsn = json.dumps(self.cfg, sort_keys=True, indent=4, 
                         separators=(',', ': '), ensure_ascii=False)
        try:
            with open(self.workspace['config'], 'wb') as outfile:
                outfile.write(jsn.encode('utf-8', 'replace'))
                lgg.info("Configuration saved")
        except Exception as why:
            lgg.warning("Write error: {0}".format(self.workspace['config']))
            lgg.warning("reason: {0}".format(why))

    def update_config(self, dct):
        """Update a version 2 of config to version 3.

        Args:
        dct -- dict(config)
        """
        idx = [None, 'fr', 'de'].index(dct['lang'])
        dct['tv_quality'] = 'HTTPS_MP4_EQ_%s' % idx
        dct['live_quality'] = 'HTTPS_EQ_1'
        dct['show_rpm'] = False
        dct['videos_folder'] = dct.pop('plus_folder', None)
        dct['music_folder'] = dct.pop('live_folder', None)
        dct['filter_keys'] = []

    def define_workspace(self):
        """Get the workspace, usually at /home/username/.Qarte

        Returns:
        dict(paths)
        """
        lgg.info('Set workspace')
        user = os.path.expanduser("~")
        paths = {}
        paths['user'] = os.path.join(user, ".Qarte")
        paths['config'] = os.path.join(paths['user'], "user_config")
        paths['live_tmp'] = os.path.join(paths['user'], "livePreviews")
        paths['tv_tmp'] = os.path.join(paths['user'], "plusPreviews")
        paths['live_stats'] = os.path.join(paths['user'], "olds.stat")
        paths['tv_summaries'] = os.path.join(paths['user'], "tv_summaries")
        paths['tv_index'] = os.path.join(paths['user'], "tv_index")
        paths['tv_videos_data'] = os.path.join(paths['user'], "tv_videos_data")
        paths['filters'] = os.path.join(paths['user'], "filter_historic")
        paths['concerts_fr'] = os.path.join(paths['user'], "concerts_fr.jsn")
        paths['concerts_de'] = os.path.join(paths['user'], "concerts_de.jsn")
        for name in ('live_tmp', 'tv_tmp'):
            if not os.path.isdir(paths[name]):
                try:
                    os.makedirs(paths[name])
                except Exception as why:
                    lgg.critical('Unable to create: %s\nReason: %s' 
                                 % (paths[name], why))
                    sys.exit()

        return paths

    def set_videos_list(self, number):
        """Display the list of arte TV videos.

        Args:
        number -- number of videos available
        """
        if self.ui:
            self.ui.arte_list.display_videos()

        self.check_video_folder()
        self.artelive.config_parser()

    def check_video_folder(self):
        """Verify if the video's folder is defined.

        """
        if self.cfg['videos_folder'] is None:
            self.display_config_dialog()

        if not os.path.isdir(self.cfg['videos_folder']):
            os.makedirs(self.cfg['videos_folder'])

    def on_tv_item_selected(self, idx):
        """Check if selection mode is interactiv with differed download
        dialog.

        Args:
        idx -- item's index
        """
        if self.artetv.is_in_category:
            self.artetv.display_category_content(idx)
            return

        item = self.artetv.videos[idx]
        lgg.info('Choose: %s' % item.title)
        if self.is_interact_with_dialog:
            if not hasattr(item, 'category'):
                self.artetv.get_stream_urls(item)
            if item.streams is not None:
                self.check_quality(item)
                self.scheduler.set_video_item(item)
                self.scheduler.dialog.show()
                self.is_interact_with_dialog = False

    def on_concert_item_selected(self, item):
        lgg.info('Choose: %s' % item.title)
        if self.is_interact_with_dialog:
            qualities = self.artelive.get_stream_urls(item)

            if qualities is None:
                lgg.warning("No stream URLs found, item rejected")
                return

            self.artelive.clean_qualities(qualities)
            item.qualities = qualities
            item.set_file_name()
            item.set_quality(self.artelive.default_quality)
            self.scheduler.set_video_item(item)
            self.scheduler.dialog.show()
            self.is_interact_with_dialog = False

    def check_quality(self, video):
        if video.quality is None:
            if self.cfg['tv_quality'] is not None:
                video.quality = self.cfg['tv_quality']

            else:
                ls = [None, 'fr', 'de']
                video.quality = "HTTP_MP4_EQ_%s" % str(ls.index(self.lang))

    def set_concerts_list(self):
        self.artelive.display_available_concerts()

    def display_config_dialog(self):
        """Show the user's preferences dialog box.

        """
        lgg.info('Call settings dialog box')
        self.settings = Settings(self)
        self.settings.show()
        self.settings.configure(self.cfg)
        self.settings.exec_()

    def reload_arte_tv(self):
        lgg.info('Reload arte TV Guide')
        # Language may be changed
        self.artetv.lang = self.cfg['lang']
        self.artetv.config_parser()

    def display_about_dialog(self):
        lgg.info('Call About dialog box')
        self.about = About(VERSION_STR)

    def create_deferred_task(self, video=None):
        lgg.info('Call deferred task dialog box: %s' % video)
        self.scheduler = LoadingScheduler(self, video)

    def close_application(self):
        """Close Qarte.

        """
        lgg.info('Prepare to exit')
        if not self.check_downloading():
            self.artetv.save_summaries_index()
            self.save_user_config()
            return True

        else:
            return False

    def check_downloading(self):
        """Return True if a downloading is still active.

        """
        if self.artetv.is_loading or self.artelive.is_loading:
            choice = self.show_warning(10, _("A downloading is active!"))
            if choice:
                self.artetv.cancel_download()
                self.artelive.cancel_download()
                return False

            else:
                return True

    def check_free_space(self, size, fld):
        stats = disk_usage(fld)
        if stats.free < (size + 2048):
            lgg.info("Not enought free space, show warning")
            self.show_space_warning(fld, size, stats.free)
            self.display_config_dialog()
            return False

        return True

    def show_warning(self, code, *args):
        """Show the warning dialog box.

        Args:
        """
        lgg.info('Warning: %s %s' %(code, "; ".join(args)))
        warn = WarningBox(code, None, self.ui)
        rep = warn.exec_()
        lgg.info('Warning return: %s' % rep)
        return rep

    def show_space_warning(self, fname, size, free):
        size = "%s bytes" % QLocale(LOC).toString(size)
        free = "%s bytes" % QLocale(LOC).toString(free)
        from gui.spacewarning import SpaceWarning
        spw = SpaceWarning(fname, size, free)
        ret = spw.exec_()

    def get_target_path(self, fld):
        """Return a temp file name builded with time.

        """
        d = str(time.time()).replace(".", "")
        t = "".join(["tv", d])
        count = 1
        while 1:
            target = os.path.join(fld, t)
            if os.path.isfile(target):
                t = "".join(["tv", d, str(count)])
                count += 1

            else:
                break

        return target


if __name__ == '__main__':
    args = sys.argv
    lvl = logging.DEBUG
    LOG_FORMAT = "%(asctime)-6s %(levelname)s: %(name)s - %(message)s"
    logging.basicConfig(format=LOG_FORMAT, datefmt='%H:%M:%S', level=lvl)
    lgg = logging.getLogger(__name__)
    app = QApplication(args)
    core = Core(args)
    sys.exit(app.exec_())
